/*
 * Copyright 2022 Fluence Labs Limited
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

mod traits;

use non_empty_vec::NonEmpty;
use serde::Deserialize;
use serde::Serialize;

pub type ResolvedLambda<'input> = NonEmpty<ResolvedValueAccessor<'input>>;

/// It's a resolved version of ValueAccessor, where FieldAccessByScalar was resolved
/// either to ArrayAccess or to FieldAccess. Intended to apply to AIR values to find out
/// an appropriate element inside them.
#[derive(Debug, PartialEq, Clone, Copy, Serialize, Deserialize)]
pub enum ResolvedValueAccessor<'input> {
    /// Access value as an array by value index.
    ArrayAccess { idx: u32 },

    /// Access value as an object by field name.
    FieldAccess { field_name: &'input str },
}
